# Make Data.r
# TWO CLASS WITH RANDOM Intercept and CORRELATED Z1 and Z2
# Sept 28, 2010
#################################;
library(mvtnorm)
library(lme4)  # Random effects regression

set.seed(092810)
 n<-1000 # sample size
 nis<-sample(1:10,n,replace=T) # number obs per subject 
 id<-rep(1:n,nis)
 N<-length(id)

# Class Indicator (as function of covariate, w)
 w<-rbinom(n,1,.5)
 W<-cbind(rep(1,n),w)
 gamma<-c(-.5,.5)
 eta<-W%*%gamma
 p<-exp(eta)/(1+exp(eta))
 c<-true.c<-rbinom(n,1,p)+1
 C<-rep(c,nis)
 n1<-length(c[c==1])
 n2<-n-n1
 N1<-sum(nis[c==1])
 N2<-N-N1

# Time Covariate: Gest Age at Visit
 x<-rep(0,N)
 for (i in 1:n) x[id==i]<-sort(44-rlnorm(nis[i],2.45,.3))
 x2<-x^2
 X<-cbind(rep(1,N),x,x2)

# Fixed-effect Coeffs
 beta1<-c(95,-.7,.01)
 beta2<-c(80,.7,-.01)

# Random effects
 sb1<-2	# Random effect SD for Class 1
 b1<-rnorm(n1,sd=sb1)

 sb2<-1	# Random effect SD for Class 1
 b2<-rnorm(n2,sd=sb2)

# Response
 y<-rep(0,N)
 sigma1<-1
 sigma2<-3
 y[C==1]<-X[C==1,]%*%beta1+rnorm(N1,0,sigma1)+rep(b1,nis[c==1])
 y[C==2]<-X[C==2,]%*%beta2+rnorm(N2,0,sigma2)+rep(b2,nis[c==2])

fit1<-lmer(y[C==1]~x[C==1]+x2[C==1]+(1|id[C==1]))
fit2<-lmer(y[C==2]~x[C==2]+x2[C==2]+(1|id[C==2]))

# Correlated Response Z1 and Z2
 cov1<-matrix(c(1,.5,.5,1),2,2)		# Class 1
 cov2<-matrix(c(1,-.5,-.5,1),2,2)
 mu1<-c(-1,1)	
 mu2<-c(.75,-.75)

 U<-matrix(0,n,2)
 U[c==1]<-rmvnorm(n1,mu1,cov1)		# Latent normal vars
 U[c==2]<-rmvnorm(n2,mu2,cov2)		# Latent normal vars
 z1<-ifelse(U[,1]<0,0,1)			# Observed binary response z1
 z2<-ifelse(U[,2]<0,0,1)			# Observed binary response z2
 
 # Joint Probs Class 1
 p001<-pmvnorm(c(-Inf,-Inf),c(0,0),mu1,cov1)  # Pr(z1=0 and z2=0), class 1
 p101<-pmvnorm(c(0,-Inf),c(Inf,0),mu1,cov1) 
 p011<-pmvnorm(c(-Inf,0),c(0,Inf),mu1,cov1) 
 p111<-pmvnorm(c(0,0),c(Inf,Inf),mu1,cov1) 

 # Joint for Class 2
 p002<-pmvnorm(c(-Inf,-Inf),c(0,0),mu2,cov2)  # Pr(z1=0 and z2=0), class 1
 p102<-pmvnorm(c(0,-Inf),c(Inf,0),mu2,cov2) 
 p012<-pmvnorm(c(-Inf,0),c(0,Inf),mu2,cov2) 
 p112<-pmvnorm(c(0,0),c(Inf,Inf),mu2,cov2) 

##############
# Store Data #
##############
data<-data.frame(id=id,y=y,x=x,z1=rep(z1,nis),z2=rep(z2,nis),W=rep(w,nis),true.C=C)

write.table(data,file="G:/staff_users/Brian/Blood Pressure/Data/data.txt",
quote=FALSE, row.names=FALSE)

