# LC02i-- TWO CLASS, CORRELATED PROBIT RANDOM INTERCEPT
# Gibbs update for muz and MH update for rho
# Infile: 	1) C:/Brian/Blood Pressure/Data/data.txt = Data
# 		      2) C:/Brian/Blood Pressure/Functions/Update Gamma.r = Update Gamma Function
#		        3) C:/Brian/Blood Pressure/Functions/Update C RanInt and Corrz.r" = Update C (latent class indicators)
#		        4) C:/Brian/Blood Pressure/Functions/Update Beta Ranint.r" = Update Beta (Fixed Effects Coefs)
#		        5) "C:/Brian/Blood Pressure/Functions/Update z MH.r" = Update biprobit parameters (e.g., mu1, mu2, rho using MH)
# TAKES ABOUT 8-10 MIN TO RUN 100O ITERATIONS WITH N = 5475
# Sept 20, 2010
################################;
library(mvtnorm)
library(Hmisc)		# for rMultinom fn used in update.c
library(MCMCpack) 	# iwish distribution used later on when updating Sigma.b
library(simex)		# For block.diag used to update z
library(msm)		# For truncated normal used to update z

source ("C:/Brian/Blood Pressure/Webpage/Update Gamma.r")
source ("C:/Brian/Blood Pressure/Webpage/Update C.r")
source ("C:/Brian/Blood Pressure/Webpage/Update Beta.r")
source ("C:/Brian/Blood Pressure/Webpage/Update z MH.r")  
				# NOTE: Update z conditional update mu1 and mu2 from full conditionals
########
# Data #
########
 data<-as.matrix(read.table("C:/Brian/Blood Pressure/Webpage/data.txt",
	header=T))
 N<-dim(data)[1]   # Total Number of Obs
 id<-data[,1]
 nis<-table(id)	# Number obs por subjecto
 n<-length(nis)	# sample size
 y<-data[,2]
 x<-data[,3]	# Gestation in Weeks
 X<-cbind(rep(1,N),x,x^2)
 z1<-z2<-w<-rep(0,n)
 tmp<-1
  for (j in 1:n) {  # one observation per subject -- Not the best code but...
  z1[j]<-data[tmp,4]
  z2[j]<-data[tmp,5]
  w[j]<-data[tmp,6]
  tmp<-tmp+nis[j]
  }
 W<-cbind(rep(1,n),w)
 r<-dim(W)[2]	# Number of class covariates (including intercept)

 K<-2			# Number of classes
 p<-ncol(X)		# Number of time parameters

############
# Num Sims #
############
 nsim<-1000		# Number of MCMC Iterations
 thin<-1
 burn<-nsim/2
 lastit<-(nsim-burn)/thin	# Last stored value

##########
# Priors #
##########	
l<-d<-.001		 # Hyperparms on tau (1/sigma2)
m0<-c(90,rep(0,p-1))	 # Prior mean and pred for beta1 and beta2
T0<-diag(.01,p)	 # Prior Precision Matrix for beta, all classes 	
gamma0<-rep(0,r)	 # Prior Mean of Gamma

#########
# Inits #
#########
 beta<-rbind(c(90,0,0),c(90,0,0))
 gamma<-matrix(0,(K-1),r)		# Class regression parms	
 sigma<-sigmab<-rep(1,K)
 tau<-1/sigma^2		
 taub<-1/sigmab^2
 c<-sample(1:K,n,replace=T)
 b<-rep(0,n)			 # Random effects
 Z<-matrix(0,N,n)			 # Random effect design matrix
 for (i in 1:n) Z[id==i,i]<-rep(1,nis[i])
 p00<-p10<-p01<-p11<-rep(.25,K)	  # Pr(z1=0 & z2=0) etc for each class
 mu<-matrix(0,K,2)			  # E(U1,U2) for each class, where U1,U2 are underlying probit var
 u2<-rep(0,n)				  # U2, Latent variable for Z2 
 tau2<-rep(1,K)			# Conditional precision of u2 given u1 for each class 
 rho<-rep(0,K)			# Corr (U1,U2) for each class

#################
# Store Samples #
#################
 Beta<-matrix(0,nrow=lastit, ncol=p*K) # Store Betas for both classes
 Sigma<-Sigmab<-matrix(0,lastit,K)		
 Gamma<-matrix(0,lastit,(K-1)*r)	
 Gtmp<-matrix(0,nsim,(K-1)*r)	# Store gamma at each it. for update gamma
 Cs<-matrix(0,lastit,n)		# Indivdual Probs + LC Indicators	
 B<-matrix(0,lastit,n)
 P00<-P10<-P01<-P11<-Mu1<-Mu2<-Rho<-matrix(0,lastit,K)	

#################
# GIBBS SAMPLER #
#################
set.seed(092010)
ptm<-proc.time()
 for (i in 1:nsim){

  # Update Gamma
    gamma<-update.gamma(g=gamma,W=W,c=c,s=2.5,gamma0=gamma0,taug=.5,
				 burn=nsim/4,it=i,G=Gtmp)
    
  # Update class indicators, c
    c<-update.c(g=gamma,y=y,z1=z1,z2=z2,nis=nis,W=W,X=X,
		    beta=beta,sigma=sigma,p00=p00,p10=p10,
		    p01=p01,p11=p11,b=b,sigmab=sigmab)
    C<-rep(c,nis)			# Concatenate to length N
    ns<-table(c)
    Ns<-table(C)	

 for (k in 1:K) {
  
  # Update beta_k
    beta[k,]<-update.beta(y=y[C==k],X=X[C==k,],b=b[c==k],nis=nis[c==k],
			    m0=m0,prec0=T0,tau=tau[k])
  
  # Update b[c==k]
    vb<-1/(taub[k]+nis[c==k]*tau[k])	
    mb<-diag(vb)%*%(tau[k]*crossprod(Z[C==k,c==k],y[C==k]-X[C==k,]%*%beta[k,]))
    b[c==k]<-rnorm(ns[k],mb,sqrt(vb))

  # Update taub_k 
    taub[k]<-rgamma(1,l+ns[k]/2,d+crossprod(b[c==k])/2)

  # Update tau_k
    zb<-rep(b[c==k],nis[c==k])
    tau[k]<-rgamma(1,l+Ns[k]/2,d+crossprod(y[C==k]-X[C==k,]%*%beta[k,]-zb)/2)
 
  # Update z1 and z2
    zresults<-update.z(y1=z1[c==k],y2=z2[c==k],mu=mu[k,],rho=rho[k],
		           u2=u2[c==k],sd=.1)
      p00[k]<-zresults$p00
	p10[k]<-zresults$p10
	p01[k]<-zresults$p01
	p11[k]<-zresults$p11
	rho[k]<-zresults$rho
      mu[k,]<-zresults$mu
      u2[c==k]<-zresults$u2
 }
 sigma<-1/sqrt(tau)
 sigmab<-1/sqrt(taub) 
   
 # Store Results
 if (i> burn & i%%thin==0) {
  j<-(i-burn)/thin
  Beta[j,]<-c(t(beta))
  Gamma[j,]<-c(t(gamma))
  Sigma[j,]<-sigma
  Sigmab[j,]<-sigmab 
  Cs[j,]<-c
  B[j,]<-b  
  P00[j,]<-p00
  P10[j,]<-p10
  P01[j,]<-p01
  P11[j,]<-p11
  Mu1[j,]<-mu[,1]
  Mu2[j,]<-mu[,2]
  Rho[j,]<-rho
 }

 Gtmp[i,]<-c(t(gamma))	# Store every it. for updt. gamma (may want to revise)
 if (i%%10==0) print(i)

}
proc.time()-ptm

###########
# Results #
###########
#NOTE: ORDER OF CLASSES CAN SWITCH FROM THOSE GENERATED IN MAKE DATA PROGRAM

mbeta<-apply(Beta,2,mean)  	# Posterior mean of betas for each class 
					# True values = 80.00  0.70 -0.01 95.00 -0.70  0.01

msigma<-apply(Sigma,2,mean)	# Error SDs -- i.e., sd of y|b --  True values = 3, 1

msigmab<-apply(Sigmab,2,mean)	# Random effect SDs -- true values = 1,3

mgamma<-apply(Gamma,2,mean)	# Posterior mean of gammas (class membership fixed effect parameters)	
					# True values = 0.50, -0.50

mrho<-apply(Rho,2,mean)		# Corr(Z1,Z2) for each class -- True value = -.5,.5

mu1<-apply(Mu1,2,mean)		# Post. Mean of U1 (latent normal of Z1) for both classes
					# True value = c(-1,1)

mu2<-apply(Mu2,2,mean)		# Post. Mean of U2 (latent normal of Z2) for both classes
					# rue value = c(.75,-.75) 

#mp00<-apply(P00,2,mean)	# Pr(Z1=0,Z2=0) for each class
#mp10<-apply(P10,2,mean) 	# Pr(Z1=1,Z2=0), etc
#mp01<-apply(P01,2,mean)
#mp11<-apply(P11,2,mean)

par(mfrow=c(2,1))
plot(1:lastit,Beta[,1],type="l",col="lightgreen")
abline(h=mean(Beta[,1]),col="blue")

plot(1:lastit,Gamma[,1],type="l",col="lightgreen")
abline(h=mean(Gamma[,1]),col="blue")

plot(1:lastit,Rho[,2],type="l",col="lightgreen")
abline(h=mean(Rho[,1]),col="blue")

