# Update C RanInt and Corr.z
# ASSUMES RANDOM INTERCEPT FOR Y CORRELATED BIRTH OUTCOMES Z1,Z2
# NOTE: MUST LOAD Hmisc Library for rMultinom function
# Function Parameters:
  # gamma = (K-1) x r matrix of class-membership reg. parameters (K and r >= 2)
  #  		 note 1: last gamma=rep(0,r) for identification purposes
  #		 note 2: if r=1 can use Dirichlet prior of class indicators
  # y = N x 1 response vector 
  # z1 = n x 1 PTB
  # z2 = n x 1 LBW
  # nis = vector of counts for each subject
  # W  = n x r matrix of covariates including intercept
  # X  = N x p matrix of time covariates (including intercept), 
  #	   SECOND COLUMN OF X IS LINEAR TIME
  # beta = K x p matrix of time coeffs
  # sigma= K x 1 vector normal SDs 
  # p00 = K x 1 vector of pr(z1=0 and z2=0) for each class
  # ... p10, p01 ...
  # p11 = K x 1 vector of pr(z1=1 and z2=1) for each class
  # b    = n x 2 matrix of random effects (***INT AND SLOPE ***)  
  # b    = n x 1 vectpr of random intercepts
  #		recall: random effects don't vary by class
  # sigmab = K x 1 random effect SD

# July 29, 2010
#######################

update.c<-function(g=gamma,y=y,z1=z1,z2=z2,nis=nis,W=W,X=X,
			 beta=beta,sigma=sigma,p00=p00,p10=p10,
			 p01=p01,p11=p11,b=b,sigmab=sigmab) {
 
 n<-nrow(W)				# Number of subjects
 r<-ncol(W)				# Number of class reg parms, incl intcpt
 N<-nrow(X)				# Total Number of Observations
 id<-rep(1:n,nis)
 if (is.vector(g)==TRUE) g<-matrix(g,1,ncol=r)	# Convert g to matrix if vector
 K<-nrow(g)+1  			# Number of classes
 
 eta<-cbind(rep(0,n),W%*%t(g))
 pi<-exp(eta)/(1+apply(as.matrix(exp(eta[,-1])),1,sum)) # Class proportions, given W

 mu<-X%*%t(beta)+rep(b,nis)	

 # Likelihood contributions, Classes 1-K
   L<-matrix(0,N,K)	# Observation-level Like
   l<-matrix(0,n,K)	# Subject-level Like
   pnum<-pz<-matrix(0,n,K)	# Numerator for post prob of C_i=k

 for (k in 1:K) {
	pz[z1==0&z2==0,k]<-p00[k]  	  # Class k like for subjects with z1=z2=0
 	pz[z1==1&z2==0,k]<-p10[k]
	pz[z1==0&z2==1,k]<-p01[k]
	pz[z1==1&z2==1,k]<-p11[k]
      L[,k]<-dnorm(y,mu[,k],sigma[k]) # Obs-level like for Y
  }
 for (i in 1:n) {
	if(nis[i]>1) l[i,]<-apply(L[id==i,],2,prod) # Subj level Likelihood
      else if(nis[i]==1) l[i,]<-L[id==i]
	if (sum(l[i,])==0) l[i,]<-rep(1,K)
 }
 
 for (k in 1:K) pnum[,k]<-pi[k]*l[,k]*pz[,k]*dnorm(b,0,sigmab[k])

 p<-pnum/(apply(pnum,1,sum))
 p[is.na(apply(p,1,sum))==T,]<-rep(1/K,K)
 c<-rMultinom(p,1)				# 1 draw for each row w.p row(p) 

 return(c)
}




