# Update Gamma (Class Regression Parameters) 
# Use Random Walk Metropolis Alogrithm for Multinomial Gen-Logit Model 
# Unknown # of Classes, K; if K=2, reduces to Logistic Reg Model
# Note: # of Regression Parms >= 2 (otherwise can use Dirichlet for C_i)

 # Function Parameters:
  # g    = (K-1) x r matrix of class-membership regr. parms (K and r >= 2)
  #  		 note 1: last gamma=rep(0,r) for identification purposes
  #		 note 2: if r=1 can use Dirichlet prior of class indicators
  # W    = n x r matrix of covariates including intercept
  # c    = n x 1 vector of current class allocations
  # gamma0 = r x 1 prior mean for gamma, all classes
  # s    = scaling factor for prior variance of gamma (i.e., S0=s*diag(r))
  # tuag = Metropolis tuning paramter
  # burn = # of burn-in iterations
  # it   = current MCMC iteration
  # G    = nsim x (K-1)*r matrix of stored gamma values (for tuning proposal density)
# Returns (K-1) x r gamma matrix
# April 18, 2009
#######################################################################

update.gamma<-function(g=gamma,W=W,c=c,gamma0=rep(0,r),
				s=100,taug=.25,burn=nsim/4,it=i,G=Gamma){
 
 n<-nrow(W)				# Number of subjects
 r<-ncol(W)				# Number of regression parms
 if (is.vector(g)==TRUE) g<-matrix(g,1,ncol=r)# Convert g to matrix if vector
 K<-nrow(g)+1  			# Number of classes	
 S0<-s*diag(r) 			# Assume diffuse prior variance
 lold<-lnew<-rep(0,K)		# Old and new likelihoods
 lpold<-lpnew<-rep(0,(K-1))	# log priors for g
 gnew<-matrix(0,K-1,r)		# Candidate g
 covk<-diag(r)			# Proposal covariance (updated after 2*burn)

 # Draw Candidate			# Note: b/c we tune cov of gnew[k,], need loop
  for (k in 1:(K-1)){		
   if (it >= 2*burn) covk<-cov(G[(burn+1):(2*burn),((k-1)*r+1):(k*r)])
   gnew[k,]<-g[k,] + rmvt(1,sigma=taug*covk,3)  	# Draw from symmetric MV t-distv
   lpold[k]<-dmvnorm(g[k,],gamma0,S0,log=T)		# Old Log Prior
   lpnew[k]<-dmvnorm(gnew[k,],gamma0,S0,log=T)	# New Log Prior
  }

 # Old and New Class Probabilities (from G-Logit Multinomial)
  etaold<-cbind(rep(0,n),W%*%t(g))
  pold<-exp(etaold)/(1+apply(as.matrix(exp(etaold[,-1])),1,sum))
  etanew<-cbind(rep(0,n),W%*%t(gnew))
  pnew<-exp(etanew)/(1+apply(as.matrix(exp(etanew[,-1])),1,sum))

 # Old and New Multinomial Likelihoods
  for (k in 1:K) {
  lold[k]<-sum(log(pold[c==k,k]))	# Class k loglike = sum[I(c_i=k)*log(p_ik)]
  lnew[k]<-sum(log(pnew[c==k,k]))
  }

 # MH Acceptance Ratio on Log Scale
   ratio<-sum(lnew)+sum(lpnew)-(sum(lold)+sum(lpold))
   if(log(runif(1))<ratio) g<-gnew
 
 return(g)

}