# Update z conditional.r
# Updates z1=PTB and z2=LBW for class k from Corr Probit Model
#
###############################################
# NOTE: This program uses marginal conditional update of mu1 then mu2|u1 
#	i.e., partitions bivariate normal for u1,u2 in marginal/conditional
# 	this speeds things up a bit compared to Metropolis
#     MUST ASSUME DIFFUSE PRIORS FOR MU1,MU2 AND U(-1,1) FOR RHO
# 	THIS IS UPDATE SHOULD BE CONSIDERED EXPERIMENTAL AND IS NOT USED 
#	IN STATS IN MEDICINE PAPER
#     
################################################
# Inputs: 	z1 and z2 = n_k x 1 binary response vectors
#		mu_k=mu1, mu2 for class k
#		rho_k = corr(U1,U2) for class k
#		u2 = underlying normals for y2
#
# Returns: 	list(p00,p10,p01,p11,mu,rho), 
#		where p00 is joint Pr(z1=0 and z2=0) etc., 
#		mu is 2x1 mean of U, # NOTE: IN PAPER, U IS Z*
#		rho is corr(U1,U2)
#		u2 = n_k x 1 vector of updated u2 values to be imported back into function
# NOTE: This function requires mvtnorm and msm packages
##################################

update.z<-function(y1=z1[c==k],y2=z2[c==k],mu=mu[k,],rho=rho[k],
			u2=u2[c==k]){
  n<-length(y1)
  n1<-sum(y1==1)
  n2<-sum(y2==1)
  u1<-rep(0,n)

  # Update U1 from Truncated Normal, given U2 and rest
    mu12<-mu[1]+rho*(u2-mu[2]) # E(U1|U2)
    s12<-sqrt(1-rho^2)	       # SD(U1|U2)
    u1[y1==0]<-rtnorm(n-n1,mu12[y1==0],s12,-Inf,0) # Need to load msm package
    u1[y1==1]<-rtnorm(n1,mu12[y1==1],s12,0,Inf)

  # Update U2 from Truncated Normal Conditional on U1 and rest
    mu21<-mu[2]+rho*(u1-mu[1]) # E(U2|U1)
    s21<-sqrt(1-rho^2)	       # SD(U2|U1) Same as s12
    u2[y2==0]<-rtnorm(n-n2,mu21[y2==0],s21,-Inf,0)  # Need to load msm package
    u2[y2==1]<-rtnorm(n2,mu21[y2==1],s21,0,Inf)

 # Update mu1 
   mu1<-rnorm(1,mean(u1),sqrt(1/n))

 # Conditional on u1, update mu2 and tau2|u1 
   tmp<-u1-mu1
   tau2<-1/(1-rho^2)
   v<-solve(n*tau2)
   m<-v%*%(tau2*sum(u2-rho*tmp))  
   mu2<-rnorm(1,m,sqrt(v))
   mu<-c(mu1,mu2)

  # Update rho 
    vrho<-1/(tau2*sum(tmp^2))  
    mrho<-vrho*(tau2*crossprod(tmp,u2-mu2)) 
    rho<-rtnorm(1,mrho,sqrt(vrho),-1,1)    # Corr(U1,U2)

 # Joint Pr(z1=0 and z2=0) etc. 
    S<-matrix(c(1,rho,rho,1),2,2)     	  # Updated Covariance
    p00<-pmvnorm(c(-Inf,-Inf),c(0,0),mu,S)  # Pr(z1=0 and z2=0)
    p10<-pmvnorm(c(0,-Inf),c(Inf,0),mu,S) 
    p01<-pmvnorm(c(-Inf,0),c(0,Inf),mu,S) 
    p11<-pmvnorm(c(0,0),c(Inf,Inf),mu,S) 

 return(list(p00=p00,p10=p10,p01=p01,p11=p11,mu=mu,rho=rho,u2=u2))
 }

